/*
 * Copyright (C) 2022  Aleksei Bavshin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef QGREETDCLIENT_HPP_
#define QGREETDCLIENT_HPP_

#include "qgreetdprotocol.hpp"

#include <QDataStream>
#include <QLocalSocket>

namespace QGreetd {

class QGreetdClient : public QObject {
    Q_OBJECT

  public:
    enum class MessageType { Info, Error };
    enum class PromptType { Visible, Secret };
    enum class State { Initial, Authenticated, Cancelled };

    Q_ENUM(MessageType);
    Q_ENUM(PromptType);
    Q_ENUM(State);

    static QString defaultPath();

    QGreetdClient(QObject *parent = nullptr);

    void connectToDaemon(QString path = defaultPath());

  Q_SIGNALS:
    void connected();
    void disconnected();
    /* Receive a raw Greetd IPC message */
    void response(const Response &);
    /* Receive processed Greetd messages */
    void showPrompt(QString text, PromptType type);
    void showMessage(QString text, MessageType type);
    void authenticationComplete();
    void done();
    void reset();

  public Q_SLOTS:
    /* Send a raw Greetd IPC message */
    void request(const Request &);
    /* Create and send a message */
    void authenticate(const QString &username);
    void respond(std::optional<QString> response = std::nullopt);
    void startSession(const QStringList &cmd, QStringList env = QStringList{});
    void cancelAuthentication();

  private Q_SLOTS:
    void readResponse();
    void handleError(QLocalSocket::LocalSocketError);
    /* Internal handler to dispatch high-level signals */
    void handleResponse(const Response &);

  private:
    template <typename T> void handleResponseImpl(const T &);

    QLocalSocket *sock;
    QDataStream stream;

    State state = State::Initial;
};

} // namespace QGreetd

#endif /* QGREETDCLIENT_HPP_ */
