/*
 * Copyright (C) 2022  Aleksei Bavshin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef QMLGREET_QGREETERCONTEXT_HPP_
#define QMLGREET_QGREETERCONTEXT_HPP_

#include "util/sessionmodel.hpp"
#include <QGreetd/Client>
#include <QObject>

namespace QmlGreet {

class QGreeterContext : public QGreetd::QGreetdClient {
    Q_OBJECT

    SessionModel *m_sessionModel;
    QSettings m_userSettings;
    QString m_hostname;
    QString m_username;
    QString m_currentCommand;
    int m_currentSession;

  public:
    Q_PROPERTY(SessionModel *sessionModel READ sessionModel CONSTANT);
    Q_PROPERTY(QString hostname READ hostname NOTIFY hostnameChanged);
    Q_PROPERTY(QString username READ username WRITE setUsername NOTIFY usernameChanged);
    Q_PROPERTY(int currentSession READ currentSession WRITE setCurrentSession NOTIFY
                   currentSessionChanged);
    Q_PROPERTY(QString currentCommand MEMBER m_currentCommand);

    QGreeterContext(QObject *parent = nullptr);

    SessionModel *sessionModel() const;
    const QString &hostname() const;

    const QString &username() const;
    void setUsername(const QString &username);

    int currentSession() const;
    void setCurrentSession(int);

    using QGreetd::QGreetdClient::startSession;
    void startSession(QModelIndex session);

  signals:
    void hostnameChanged(const QString &);
    void usernameChanged(const QString &);
    void currentSessionChanged(int);

  public slots:
    void respond(QString response);

  private slots:
    void onAuthenticationComplete();

  private:
    void loadState();
};
} // namespace QmlGreet

#endif /* QMLGREET_QGREETERCONTEXT_HPP_ */
