/*
 * Copyright (C) 2022  Aleksei Bavshin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"
#include "qmlgreeter.hpp"
#include "util/configuration.hpp"

#include <QGuiApplication>
#include <QLocale>
#include <QTranslator>

int main(int argc, char *argv[]) {
    qputenv("QT_NO_XDG_DESKTOP_PORTAL", "1");
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
#endif
    QGuiApplication app(argc, argv);
    QGuiApplication::setApplicationName("qmlgreet");
    QGuiApplication::setApplicationVersion(QMLGREET_VERSION);

    auto &parser = QmlGreet::Configuration::createCommandLineParser();
    parser.addHelpOption();
    parser.addVersionOption();
    parser.process(app);

    QTranslator translator;
    const QStringList uiLanguages = QLocale::system().uiLanguages();
    for (const QString &locale : uiLanguages) {
        const QString baseName = "qmlgreet_" + QLocale(locale).name();
        if (translator.load(":/i18n/" + baseName)) {
            app.installTranslator(&translator);
            break;
        }
    }

    QmlGreet::QmlGreeter *greeter = new QmlGreet::QmlGreeter(&app);
    QTimer::singleShot(0, greeter, SLOT(start()));

    return app.exec();
}
