/*
 * Copyright (C) 2022  Aleksei Bavshin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef QMLGREET_UTIL_CONFIGURATION_HPP_
#define QMLGREET_UTIL_CONFIGURATION_HPP_

#include <QtCore>

QT_BEGIN_NAMESPACE
class QCommandLineParser;
QT_END_NAMESPACE

namespace QmlGreet {

/* Helper to clean up QSettings Group automatically upon scope exit */
class QSettingsGroupScope {
  public:
    QSettingsGroupScope(QSettings &settings, const QString &group);
    ~QSettingsGroupScope();

  private:
    QSettings &m_settings;
    QString m_group;
};

/* acceptable types: bool, QString, QStringList */
class Option {
    using ValueType = QVariant;

  public:
    QString section;
    QString key;
    QCommandLineOption option;
    ValueType defaultValue;

    Option(QString &&section, QString &&key, QCommandLineOption &&option, ValueType defaultValue)
        : section{section}, key{key}, option{option}, defaultValue{defaultValue} {}

    template <typename VT = QVariant> VT get() const;

    QStringList getList(QLatin1Char separator = QLatin1Char(',')) const;

    operator bool() const;
};

template <> QVariant Option::get<QVariant>() const;
template <> QString Option::get<QString>() const;
template <> bool Option::get<bool>() const;

class Configuration {
  public:
    static Option &registerOption(Option &&opt);

    static QCommandLineParser &createCommandLineParser();

    static QVariant value(Option &);
    static QVariant value(const QString &key, const QVariant &defaultValue = QVariant());
};
} // namespace QmlGreet

#define QMLGREET_DECLARE_OPTION(section, key, cliopt, defaultValue)                                \
    namespace QmlGreet::Settings::section {                                                        \
    static const ::QmlGreet::Option &key =                                                         \
        ::QmlGreet::Configuration::registerOption({#section, #key, cliopt, defaultValue});         \
    }

#endif /* QMLGREET_UTIL_CONFIGURATION_HPP_ */
