/*
 * Copyright (C) 2022  Aleksei Bavshin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef QMLGREET_UTIL_SESSIONMODEL_HPP_
#define QMLGREET_UTIL_SESSIONMODEL_HPP_

#include <QAbstractListModel>
#include <QDir>
#include <QString>

namespace QmlGreet {
Q_NAMESPACE

class SessionModel : public QAbstractListModel {
    Q_OBJECT

  public:
    enum SessionModelRoles {
        KeyRole = Qt::UserRole,
        ExecRole,
        TypeRole,
    };

    enum SessionType {
        Wayland,
        X11,
        TTY,
    };

    Q_ENUM(SessionType);

    SessionModel(QObject *parent = nullptr);

    QModelIndex findSession(SessionType type, const QString &key);
    void loadSession(SessionType type, const QFileInfo &location);
    void loadSessions(SessionType type, const QDir &location);

    QHash<int, QByteArray> roleNames() const override;
    int rowCount(const QModelIndex &parent) const override;
    QVariant data(const QModelIndex &index, int role) const override;

  private:
    struct ItemType {
        SessionType type;
        QString key;
        QString name;
        QString comment;
        QString exec;
    };

    QList<ItemType> m_items;
    QList<ItemType> m_hidden;
};

} // namespace QmlGreet

#endif /* QMLGREET_UTIL_SESSIONMODEL_HPP_ */
